"""Test usage statistics transmission client-side."""
import copy
from typing import Any, Dict, List

import pytest
import requests

from great_expectations.core.usage_statistics.anonymizers.types.base import (
    GETTING_STARTED_DATASOURCE_NAME,
    CLISuiteInteractiveFlagCombinations,
)
from great_expectations.core.usage_statistics.events import UsageStatsEvents
from great_expectations.data_context.config_validator.yaml_config_validator import (
    _YamlConfigValidator,
)
from tests.integration.usage_statistics.example_events.cloud_migrate_example_events import (
    cloud_migrate,
)
from tests.integration.usage_statistics.example_events.data_context_run_validation_operator import (
    data_context_run_validation_operator_events,
)
from tests.integration.usage_statistics.test_integration_usage_statistics import (
    USAGE_STATISTICS_QA_URL,
)
from tests.integration.usage_statistics.usage_stats_event_examples import (
    data_context_init_with_dependencies,
)


def generate_messages_with_defaults(
    defaults: Dict[str, Any], message_stubs: List[Dict[str, Any]]
) -> List[Dict[str, Any]]:
    """
    Create a list of messages by overriding defaults with message_stubs
    Args:
        defaults: Dict of default message items
        message_stubs: Unique parts of message

    Returns:
        List of messages same len(message_stubs) combining defaults overridden by message stubs
    """
    output_list = []
    for message_stub in message_stubs:
        defaults_copy = copy.deepcopy(defaults)
        defaults_copy.update(message_stub)
        output_list.append(defaults_copy)

    return output_list


def test_generate_messages_with_defaults():

    defaults = {
        "success": True,
        "version": "1.0.0",
        "event_time": "2020-08-04T22:50:58.837Z",
        "data_context_id": "00000000-0000-0000-0000-000000000002",
        "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
    }

    message_stubs = [
        {
            "event": "cli.checkpoint.new",
            "event_payload": {},
            "ge_version": "0.11.9.manual_testing",
        },
        {
            "event": "cli.checkpoint.new",
            "event_payload": {"api_version": "v2"},
            "ge_version": "0.13.0.manual_testing",
        },
        {
            "event": "cli.checkpoint.new",
            "event_payload": {"api_version": "v3"},
            "ge_version": "0.13.0.manual_testing",
        },
        {
            "event": "cli.checkpoint.new.begin",
            "event_payload": {"api_version": "v3"},
            "ge_version": "0.13.18.manual_testing",
        },
        {
            "event": "cli.checkpoint.new.end",
            "event_payload": {"api_version": "v3"},
            "ge_version": "0.13.18.manual_testing",
        },
    ]

    output = generate_messages_with_defaults(
        defaults=defaults, message_stubs=message_stubs
    )

    expected = [
        {
            "event": "cli.checkpoint.new",
            "event_payload": {},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T22:50:58.837Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.11.9.manual_testing",
        },
        {
            "event": "cli.checkpoint.new",
            "event_payload": {"api_version": "v2"},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T22:50:58.837Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.0.manual_testing",
        },
        {
            "event": "cli.checkpoint.new",
            "event_payload": {"api_version": "v3"},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T22:50:58.837Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.0.manual_testing",
        },
        {
            "event": "cli.checkpoint.new.begin",
            "event_payload": {"api_version": "v3"},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T22:50:58.837Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.18.manual_testing",
        },
        {
            "event": "cli.checkpoint.new.end",
            "event_payload": {"api_version": "v3"},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T22:50:58.837Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.18.manual_testing",
        },
    ]

    assert output == expected


"""
valid_usage_statistics_messages should include a list of messages that we want to ensure are valid.

Whenever a new kind of message is added, an example of that message should be included here.

Each message will be sent to the server to ensure it is accepted.

"""
valid_usage_statistics_messages = {
    "data_context.__init__": [
        {
            "event_payload": {
                "platform.system": "Darwin",
                "platform.release": "19.3.0",
                "version_info": "sys.version_info(major=3, minor=7, micro=4, releaselevel='final', serial=0)",
                "anonymized_datasources": [
                    {
                        "anonymized_name": "f57d8a6edae4f321b833384801847498",
                        "parent_class": "SqlAlchemyDatasource",
                        "sqlalchemy_dialect": "postgresql",
                    },
                ],
                "anonymized_stores": [
                    {
                        "anonymized_name": "078eceafc1051edf98ae2f911484c7f7",
                        "parent_class": "ExpectationsStore",
                        "anonymized_store_backend": {
                            "parent_class": "TupleFilesystemStoreBackend"
                        },
                    },
                    {
                        "anonymized_name": "313cbd9858dd92f3fc2ef1c10ab9c7c8",
                        "parent_class": "ValidationsStore",
                        "anonymized_store_backend": {
                            "parent_class": "TupleFilesystemStoreBackend"
                        },
                    },
                    {
                        "anonymized_name": "2d487386aa7b39e00ed672739421473f",
                        "parent_class": "EvaluationParameterStore",
                        "anonymized_store_backend": {
                            "parent_class": "InMemoryStoreBackend"
                        },
                    },
                ],
                "anonymized_validation_operators": [
                    {
                        "anonymized_name": "99d14cc00b69317551690fb8a61aca94",
                        "parent_class": "ActionListValidationOperator",
                        "anonymized_action_list": [
                            {
                                "anonymized_name": "5a170e5b77c092cc6c9f5cf2b639459a",
                                "parent_class": "StoreValidationResultAction",
                            },
                            {
                                "anonymized_name": "0fffe1906a8f2a5625a5659a848c25a3",
                                "parent_class": "StoreEvaluationParametersAction",
                            },
                            {
                                "anonymized_name": "101c746ab7597e22b94d6e5f10b75916",
                                "parent_class": "UpdateDataDocsAction",
                            },
                        ],
                    },
                ],
                "anonymized_data_docs_sites": [
                    {
                        "parent_class": "SiteBuilder",
                        "anonymized_name": "eaf0cf17ad63abf1477f7c37ad192700",
                        "anonymized_store_backend": {
                            "parent_class": "TupleFilesystemStoreBackend"
                        },
                        "anonymized_site_index_builder": {
                            "parent_class": "DefaultSiteIndexBuilder",
                        },
                    },
                ],
                "anonymized_expectation_suites": [
                    {
                        "anonymized_name": "238e99998c7674e4ff26a9c529d43da4",
                        "expectation_count": 8,
                        "anonymized_expectation_type_counts": {
                            "expect_column_value_lengths_to_be_between": 1,
                            "expect_table_row_count_to_be_between": 1,
                            "expect_column_values_to_not_be_null": 2,
                            "expect_column_distinct_values_to_be_in_set": 1,
                            "expect_column_kl_divergence_to_be_less_than": 1,
                            "expect_table_column_count_to_equal": 1,
                            "expect_table_columns_to_match_ordered_list": 1,
                        },
                    },
                ],
            },
            "event": "data_context.__init__",
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-03-28T01:14:21.155Z",
            "data_context_id": "96c547fe-e809-4f2e-b122-0dc91bb7b3ad",
            "data_context_instance_id": "445a8ad1-2bd0-45ce-bb6b-d066afe996dd",
            "ge_version": "0.11.9.manual_test",
        },
        # "new-style" expectation type system
        {
            "event_payload": {
                "platform.system": "Darwin",
                "platform.release": "19.3.0",
                "version_info": "sys.version_info(major=3, minor=7, micro=4, releaselevel='final', serial=0)",
                "anonymized_datasources": [
                    {
                        "anonymized_name": "f57d8a6edae4f321b833384801847498",
                        "parent_class": "SqlAlchemyDatasource",
                        "sqlalchemy_dialect": "postgresql",
                    },
                ],
                "anonymized_stores": [
                    {
                        "anonymized_name": "078eceafc1051edf98ae2f911484c7f7",
                        "parent_class": "ExpectationsStore",
                        "anonymized_store_backend": {
                            "parent_class": "TupleFilesystemStoreBackend"
                        },
                    },
                    {
                        "anonymized_name": "313cbd9858dd92f3fc2ef1c10ab9c7c8",
                        "parent_class": "ValidationsStore",
                        "anonymized_store_backend": {
                            "parent_class": "TupleFilesystemStoreBackend"
                        },
                    },
                    {
                        "anonymized_name": "2d487386aa7b39e00ed672739421473f",
                        "parent_class": "EvaluationParameterStore",
                        "anonymized_store_backend": {
                            "parent_class": "InMemoryStoreBackend"
                        },
                    },
                ],
                "anonymized_validation_operators": [
                    {
                        "anonymized_name": "99d14cc00b69317551690fb8a61aca94",
                        "parent_class": "ActionListValidationOperator",
                        "anonymized_action_list": [
                            {
                                "anonymized_name": "5a170e5b77c092cc6c9f5cf2b639459a",
                                "parent_class": "StoreValidationResultAction",
                            },
                            {
                                "anonymized_name": "0fffe1906a8f2a5625a5659a848c25a3",
                                "parent_class": "StoreEvaluationParametersAction",
                            },
                            {
                                "anonymized_name": "101c746ab7597e22b94d6e5f10b75916",
                                "parent_class": "UpdateDataDocsAction",
                            },
                        ],
                    },
                ],
                "anonymized_data_docs_sites": [
                    {
                        "parent_class": "SiteBuilder",
                        "anonymized_name": "eaf0cf17ad63abf1477f7c37ad192700",
                        "anonymized_store_backend": {
                            "parent_class": "TupleFilesystemStoreBackend"
                        },
                        "anonymized_site_index_builder": {
                            "parent_class": "DefaultSiteIndexBuilder",
                        },
                    },
                ],
                "anonymized_expectation_suites": [
                    {
                        "anonymized_name": "238e99998c7674e4ff26a9c529d43da4",
                        "expectation_count": 8,
                        "anonymized_expectation_counts": [
                            {
                                "expectation_type": "expect_column_value_lengths_to_be_between",
                                "count": 1,
                            },
                            {
                                "expectation_type": "expect_table_row_count_to_be_between",
                                "count": 1,
                            },
                            {
                                "expectation_type": "expect_column_values_to_not_be_null",
                                "count": 2,
                            },
                            {
                                "expectation_type": "expect_column_distinct_values_to_be_in_set",
                                "count": 1,
                            },
                            {
                                "expectation_type": "expect_column_kl_divergence_to_be_less_than",
                                "count": 1,
                            },
                            {
                                "expectation_type": "expect_table_column_count_to_equal",
                                "count": 1,
                            },
                            {
                                "expectation_type": "expect_table_columns_to_match_ordered_list",
                                "count": 1,
                            },
                        ],
                    },
                ],
            },
            "event": "data_context.__init__",
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-03-28T01:14:21.155Z",
            "data_context_id": "96c547fe-e809-4f2e-b122-0dc91bb7b3ad",
            "data_context_instance_id": "445a8ad1-2bd0-45ce-bb6b-d066afe996dd",
            "ge_version": "0.13.0.manual_test",
        },
        data_context_init_with_dependencies,
    ],
    "data_asset.validate": [
        {
            "event": "data_asset.validate",
            "event_payload": {
                "anonymized_batch_kwarg_keys": [
                    "path",
                    "datasource",
                    "data_asset_name",
                ],
                "anonymized_expectation_suite_name": "dbb859464809a03647feb14a514f12b8",
                "anonymized_datasource_name": "a41caeac7edb993cfbe55746e6a328b5",
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-03T23:36:26.422Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.11.9.manual_testing",
        },
    ],
    "data_context.add_datasource": [
        {
            "event": "data_context.add_datasource",
            "event_payload": {
                "anonymized_name": "c9633f65c36d1ba9fbaa9009c1404cfa",
                "parent_class": "PandasDatasource",
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:16.030Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.11.9.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
    ],
    "data_context.get_batch_list": [
        {
            "event": "data_context.get_batch_list",
            "event_payload": {
                "anonymized_batch_request_required_top_level_properties": {
                    "anonymized_datasource_name": GETTING_STARTED_DATASOURCE_NAME,
                    "anonymized_data_connector_name": "af09acd176f54642635a8a2975305437",
                    "anonymized_data_asset_name": "9104abd890c05a364f379443b9f43825",
                },
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:16.030Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "data_context.get_batch_list",
            "event_payload": {
                "anonymized_batch_request_required_top_level_properties": {
                    "anonymized_datasource_name": GETTING_STARTED_DATASOURCE_NAME,
                    "anonymized_data_connector_name": "af09acd176f54642635a8a2975305437",
                    "anonymized_data_asset_name": "38b9086d45a8746d014a0d63ad58e331",
                },
                "batch_request_optional_top_level_keys": ["data_connector_query"],
                "data_connector_query_keys": ["index"],
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:16.030Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "data_context.get_batch_list",
            "event_payload": {
                "anonymized_batch_request_required_top_level_properties": {
                    "anonymized_datasource_name": GETTING_STARTED_DATASOURCE_NAME,
                    "anonymized_data_connector_name": "e475f70ca0bcbaf2748b93da5e9867ec",
                    "anonymized_data_asset_name": "2621a5230efeef1973ff373dd12b1ac4",
                },
                "batch_request_optional_top_level_keys": ["batch_spec_passthrough"],
                "batch_spec_passthrough_keys": ["reader_options"],
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:16.030Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "data_context.get_batch_list",
            "event_payload": {
                "anonymized_batch_request_required_top_level_properties": {
                    "anonymized_datasource_name": "a732a247720783a5931fa7c4606403c2",
                    "anonymized_data_connector_name": "e475f70ca0bcbaf2748b93da5e9867ec",
                    "anonymized_data_asset_name": "2621a5230efeef1973ff373dd12b1ac4",
                },
                "batch_request_optional_top_level_keys": [
                    "data_connector_query",
                    "batch_spec_passthrough",
                ],
                "batch_spec_passthrough_keys": ["reader_options"],
                "data_connector_query_keys": ["index"],
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:16.030Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "data_context.get_batch_list",
            "event_payload": {
                "anonymized_batch_request_required_top_level_properties": {
                    "anonymized_datasource_name": "a732a247720783a5931fa7c4606403c2",
                    "anonymized_data_connector_name": "af09acd176f54642635a8a2975305437",
                    "anonymized_data_asset_name": "38b9086d45a8746d014a0d63ad58e331",
                },
                "batch_request_optional_top_level_keys": [
                    "data_connector_query",
                    "batch_spec_passthrough",
                ],
                "batch_spec_passthrough_keys": ["reader_method"],
                "data_connector_query_keys": ["index"],
                "runtime_parameters_keys": ["path"],
            },
            "success": False,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:16.030Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "data_context.get_batch_list",
            "event_payload": {
                "anonymized_batch_request_required_top_level_properties": {
                    "anonymized_datasource_name": "a732a247720783a5931fa7c4606403c2",
                    "anonymized_data_connector_name": "af09acd176f54642635a8a2975305437",
                    "anonymized_data_asset_name": "38b9086d45a8746d014a0d63ad58e331",
                },
                "batch_request_optional_top_level_keys": [
                    "batch_spec_passthrough",
                    "data_connector_query",
                ],
                "data_connector_query_keys": ["index"],
                "runtime_parameters_keys": ["path"],
                "batch_spec_passthrough_keys": ["reader_method", "reader_options"],
            },
            "success": False,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:16.030Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "data_context.get_batch_list",
            "event_payload": {
                "anonymized_batch_request_required_top_level_properties": {
                    "anonymized_datasource_name": GETTING_STARTED_DATASOURCE_NAME,
                    "anonymized_data_connector_name": "d52d7bff3226a7f94dd3510c1040de78",
                    "anonymized_data_asset_name": "556e8c06239d09fc66f424eabb9ca491",
                },
                "batch_request_optional_top_level_keys": [
                    "batch_identifiers",
                    "runtime_parameters",
                ],
                "runtime_parameters_keys": ["batch_data"],
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:28.070Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "data_context.get_batch_list",
            "event_payload": {
                "anonymized_batch_request_required_top_level_properties": {
                    "anonymized_datasource_name": "a732a247720783a5931fa7c4606403c2",
                    "anonymized_data_connector_name": "d52d7bff3226a7f94dd3510c1040de78",
                    "anonymized_data_asset_name": "556e8c06239d09fc66f424eabb9ca491",
                },
                "batch_request_optional_top_level_keys": [
                    "batch_identifiers",
                    "runtime_parameters",
                ],
                "runtime_parameters_keys": ["batch_data"],
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:28.070Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
    ],
    "data_context.build_data_docs": [
        {
            "event_payload": {},
            "event": "data_context.build_data_docs",
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:24.349Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.11.9.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
    ],
    "data_context.open_data_docs": [
        {
            "event_payload": {},
            "event": "data_context.open_data_docs",
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:28.070Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.11.9.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
    ],
    "data_context.run_checkpoint": [
        {
            "event": "data_context.run_checkpoint",
            "event_payload": {},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:28.070Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "data_context.run_checkpoint",
            "event_payload": {},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:28.070Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
    ],
    "checkpoint.run": [
        {
            "event": "checkpoint.run",
            "event_payload": {},
            "success": False,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:28.070Z",
            "event_duration": 1,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "checkpoint.run",
            "event_payload": {
                "anonymized_name": "48533197103a407af37326b0224a97df",
                "config_version": 1,
                "anonymized_run_name_template": "21e9677f05fd2b0d83bb9285a688d5c5",
                "anonymized_expectation_suite_name": "4987b41d9e7012f6a86a8b3939739eff",
                "anonymized_action_list": [
                    {
                        "anonymized_name": "8e3e134cd0402c3970a02f40d2edfc26",
                        "parent_class": "StoreValidationResultAction",
                    },
                    {
                        "anonymized_name": "40e24f0c6b04b6d4657147990d6f39bd",
                        "parent_class": "StoreEvaluationParametersAction",
                    },
                    {
                        "anonymized_name": "2b99b6b280b8a6ad1176f37580a16411",
                        "parent_class": "UpdateDataDocsAction",
                    },
                ],
                "anonymized_validations": [
                    {
                        "anonymized_batch_request": {
                            "anonymized_batch_request_required_top_level_properties": {
                                "anonymized_datasource_name": "41cc60fba42f099f878a4bb295dc08c9",
                                "anonymized_data_connector_name": "4cffb49069fa5fececc8032aa41ff791",
                                "anonymized_data_asset_name": "5dce9f4b8abd8adbb4f719e05fceecab",
                            },
                            "batch_request_optional_top_level_keys": [
                                "data_connector_query"
                            ],
                        },
                        "anonymized_expectation_suite_name": "4987b41d9e7012f6a86a8b3939739eff",
                        "anonymized_action_list": [
                            {
                                "anonymized_name": "8e3e134cd0402c3970a02f40d2edfc26",
                                "parent_class": "StoreValidationResultAction",
                            },
                            {
                                "anonymized_name": "40e24f0c6b04b6d4657147990d6f39bd",
                                "parent_class": "StoreEvaluationParametersAction",
                            },
                            {
                                "anonymized_name": "2b99b6b280b8a6ad1176f37580a16411",
                                "parent_class": "UpdateDataDocsAction",
                            },
                        ],
                    }
                ],
                "checkpoint_optional_top_level_keys": [
                    "evaluation_parameters",
                    "runtime_configuration",
                ],
            },
            "success": False,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:28.070Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "checkpoint.run",
            "event_payload": {
                "anonymized_name": "eb2d802f924a3e764afc605de3495c5c",
                "config_version": 1.0,
                "anonymized_run_name_template": "21e9677f05fd2b0d83bb9285a688d5c5",
                "anonymized_validations": [
                    {
                        "anonymized_batch_request": {
                            "anonymized_batch_request_required_top_level_properties": {
                                "anonymized_datasource_name": "a732a247720783a5931fa7c4606403c2",
                                "anonymized_data_connector_name": "e475f70ca0bcbaf2748b93da5e9867ec",
                                "anonymized_data_asset_name": "2621a5230efeef1973ff373dd12b1ac4",
                            },
                            "batch_request_optional_top_level_keys": [
                                "data_connector_query"
                            ],
                            "data_connector_query_keys": ["index"],
                        },
                        "anonymized_expectation_suite_name": "35af1ba156bfe672f8845cb60554b138",
                        "anonymized_action_list": [
                            {
                                "anonymized_name": "8e3e134cd0402c3970a02f40d2edfc26",
                                "parent_class": "StoreValidationResultAction",
                            },
                            {
                                "anonymized_name": "40e24f0c6b04b6d4657147990d6f39bd",
                                "parent_class": "StoreEvaluationParametersAction",
                            },
                            {
                                "anonymized_name": "2b99b6b280b8a6ad1176f37580a16411",
                                "parent_class": "UpdateDataDocsAction",
                            },
                        ],
                    }
                ],
                "checkpoint_optional_top_level_keys": [
                    "evaluation_parameters",
                    "runtime_configuration",
                ],
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:28.070Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "checkpoint.run",
            "event_payload": {
                "anonymized_name": "eb2d802f924a3e764afc605de3495c5c",
                "config_version": 1.0,
                "anonymized_run_name_template": "21e9677f05fd2b0d83bb9285a688d5c5",
                "anonymized_validations": [
                    {
                        "anonymized_batch_request": {
                            "anonymized_batch_request_required_top_level_properties": {
                                "anonymized_datasource_name": "getting_started_datasource",
                                "anonymized_data_connector_name": "af09acd176f54642635a8a2975305437",
                                "anonymized_data_asset_name": "9104abd890c05a364f379443b9f43825",
                            },
                            "batch_request_optional_top_level_keys": [
                                "batch_spec_passthrough",
                                "data_connector_query",
                            ],
                            "batch_spec_passthrough_keys": ["reader_options"],
                        },
                        "anonymized_expectation_suite_name": "35af1ba156bfe672f8845cb60554b138",
                        "anonymized_action_list": [
                            {
                                "anonymized_name": "8e3e134cd0402c3970a02f40d2edfc26",
                                "parent_class": "StoreValidationResultAction",
                            },
                            {
                                "anonymized_name": "40e24f0c6b04b6d4657147990d6f39bd",
                                "parent_class": "StoreEvaluationParametersAction",
                            },
                            {
                                "anonymized_name": "2b99b6b280b8a6ad1176f37580a16411",
                                "parent_class": "UpdateDataDocsAction",
                            },
                        ],
                    }
                ],
                "checkpoint_optional_top_level_keys": [
                    "evaluation_parameters",
                    "runtime_configuration",
                ],
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:28.070Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "checkpoint.run",
            "event_payload": {
                "anonymized_name": "f563d9aa1604e16099bb7dff7b203319",
                "config_version": 1.0,
                "anonymized_expectation_suite_name": "6a04fc37da0d43a4c21429f6788d2cff",
                "anonymized_action_list": [
                    {
                        "anonymized_name": "8e3e134cd0402c3970a02f40d2edfc26",
                        "parent_class": "StoreValidationResultAction",
                    },
                    {
                        "anonymized_name": "40e24f0c6b04b6d4657147990d6f39bd",
                        "parent_class": "StoreEvaluationParametersAction",
                    },
                    {
                        "anonymized_name": "2b99b6b280b8a6ad1176f37580a16411",
                        "parent_class": "UpdateDataDocsAction",
                    },
                ],
                "anonymized_validations": [
                    {
                        "anonymized_batch_request": {
                            "anonymized_batch_request_required_top_level_properties": {
                                "anonymized_datasource_name": "a732a247720783a5931fa7c4606403c2",
                                "anonymized_data_connector_name": "d52d7bff3226a7f94dd3510c1040de78",
                                "anonymized_data_asset_name": "556e8c06239d09fc66f424eabb9ca491",
                            },
                            "batch_request_optional_top_level_keys": [
                                "batch_identifiers",
                                "runtime_parameters",
                            ],
                            "runtime_parameters_keys": ["batch_data"],
                        },
                        "anonymized_expectation_suite_name": "6a04fc37da0d43a4c21429f6788d2cff",
                        "anonymized_action_list": [
                            {
                                "anonymized_name": "8e3e134cd0402c3970a02f40d2edfc26",
                                "parent_class": "StoreValidationResultAction",
                            },
                            {
                                "anonymized_name": "40e24f0c6b04b6d4657147990d6f39bd",
                                "parent_class": "StoreEvaluationParametersAction",
                            },
                            {
                                "anonymized_name": "2b99b6b280b8a6ad1176f37580a16411",
                                "parent_class": "UpdateDataDocsAction",
                            },
                        ],
                    },
                ],
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:28.070Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "checkpoint.run",
            "event_payload": {
                "anonymized_name": "f563d9aa1604e16099bb7dff7b203319",
                "config_version": 1.0,
                "anonymized_expectation_suite_name": "6a04fc37da0d43a4c21429f6788d2cff",
                "anonymized_action_list": [
                    {
                        "anonymized_name": "8e3e134cd0402c3970a02f40d2edfc26",
                        "parent_class": "StoreValidationResultAction",
                    },
                    {
                        "anonymized_name": "40e24f0c6b04b6d4657147990d6f39bd",
                        "parent_class": "StoreEvaluationParametersAction",
                    },
                    {
                        "anonymized_name": "2b99b6b280b8a6ad1176f37580a16411",
                        "parent_class": "UpdateDataDocsAction",
                    },
                ],
                "anonymized_validations": [
                    {
                        "anonymized_batch_request": {
                            "anonymized_batch_request_required_top_level_properties": {
                                "anonymized_datasource_name": GETTING_STARTED_DATASOURCE_NAME,
                                "anonymized_data_connector_name": "d52d7bff3226a7f94dd3510c1040de78",
                                "anonymized_data_asset_name": "556e8c06239d09fc66f424eabb9ca491",
                            },
                            "batch_request_optional_top_level_keys": [
                                "batch_identifiers",
                                "runtime_parameters",
                            ],
                            "runtime_parameters_keys": ["batch_data"],
                        },
                        "anonymized_expectation_suite_name": "6a04fc37da0d43a4c21429f6788d2cff",
                        "anonymized_action_list": [
                            {
                                "anonymized_name": "8e3e134cd0402c3970a02f40d2edfc26",
                                "parent_class": "StoreValidationResultAction",
                            },
                            {
                                "anonymized_name": "40e24f0c6b04b6d4657147990d6f39bd",
                                "parent_class": "StoreEvaluationParametersAction",
                            },
                            {
                                "anonymized_name": "2b99b6b280b8a6ad1176f37580a16411",
                                "parent_class": "UpdateDataDocsAction",
                            },
                        ],
                    },
                ],
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:28.070Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "checkpoint.run",
            "event_payload": {
                "anonymized_name": "d7e22c0913c0cb83d528d2a7ad097f2b",
                "config_version": 1.0,
                "anonymized_run_name_template": "131f67e5ea07d59f2bc5376234f7f9f2",
                "anonymized_expectation_suite_name": "295722d0683963209e24034a79235ba6",
                "anonymized_action_list": [
                    {
                        "anonymized_name": "8e3e134cd0402c3970a02f40d2edfc26",
                        "parent_class": "StoreValidationResultAction",
                    },
                    {
                        "anonymized_name": "40e24f0c6b04b6d4657147990d6f39bd",
                        "parent_class": "StoreEvaluationParametersAction",
                    },
                    {
                        "anonymized_name": "2b99b6b280b8a6ad1176f37580a16411",
                        "parent_class": "UpdateDataDocsAction",
                    },
                ],
                "anonymized_validations": [
                    {
                        "anonymized_batch_request": {
                            "anonymized_batch_request_required_top_level_properties": {
                                "anonymized_datasource_name": "a732a247720783a5931fa7c4606403c2",
                                "anonymized_data_connector_name": "af09acd176f54642635a8a2975305437",
                                "anonymized_data_asset_name": "38b9086d45a8746d014a0d63ad58e331",
                            },
                            "batch_request_optional_top_level_keys": [
                                "batch_identifiers",
                                "data_connector_query",
                                "runtime_parameters",
                            ],
                            "runtime_parameters_keys": ["batch_data"],
                        },
                        "anonymized_expectation_suite_name": "295722d0683963209e24034a79235ba6",
                        "anonymized_action_list": [
                            {
                                "anonymized_name": "8e3e134cd0402c3970a02f40d2edfc26",
                                "parent_class": "StoreValidationResultAction",
                            },
                            {
                                "anonymized_name": "40e24f0c6b04b6d4657147990d6f39bd",
                                "parent_class": "StoreEvaluationParametersAction",
                            },
                            {
                                "anonymized_name": "2b99b6b280b8a6ad1176f37580a16411",
                                "parent_class": "UpdateDataDocsAction",
                            },
                        ],
                    },
                    {
                        "anonymized_batch_request": {
                            "anonymized_batch_request_required_top_level_properties": {
                                "anonymized_datasource_name": "a732a247720783a5931fa7c4606403c2",
                                "anonymized_data_connector_name": "af09acd176f54642635a8a2975305437",
                                "anonymized_data_asset_name": "38b9086d45a8746d014a0d63ad58e331",
                            },
                            "batch_request_optional_top_level_keys": [
                                "batch_identifiers",
                                "data_connector_query",
                                "runtime_parameters",
                            ],
                            "runtime_parameters_keys": ["batch_data"],
                        },
                        "anonymized_expectation_suite_name": "295722d0683963209e24034a79235ba6",
                        "anonymized_action_list": [
                            {
                                "anonymized_name": "8e3e134cd0402c3970a02f40d2edfc26",
                                "parent_class": "StoreValidationResultAction",
                            },
                            {
                                "anonymized_name": "40e24f0c6b04b6d4657147990d6f39bd",
                                "parent_class": "StoreEvaluationParametersAction",
                            },
                            {
                                "anonymized_name": "2b99b6b280b8a6ad1176f37580a16411",
                                "parent_class": "UpdateDataDocsAction",
                            },
                        ],
                    },
                ],
            },
            "success": False,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:28.070Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "checkpoint.run",
            "event_payload": {
                "anonymized_name": "f563d9aa1604e16099bb7dff7b203319",
                "config_version": 1.0,
                "anonymized_validations": [
                    {
                        "anonymized_batch_request": {
                            "anonymized_batch_request_required_top_level_properties": {
                                "anonymized_datasource_name": GETTING_STARTED_DATASOURCE_NAME,
                                "anonymized_data_connector_name": "d52d7bff3226a7f94dd3510c1040de78",
                                "anonymized_data_asset_name": "556e8c06239d09fc66f424eabb9ca491",
                            },
                            "batch_request_optional_top_level_keys": [
                                "runtime_parameters",
                            ],
                            "runtime_parameters_keys": ["query"],
                        },
                        "anonymized_expectation_suite_name": "6a04fc37da0d43a4c21429f6788d2cff",
                        "anonymized_action_list": [
                            {
                                "anonymized_name": "8e3e134cd0402c3970a02f40d2edfc26",
                                "parent_class": "StoreValidationResultAction",
                            },
                            {
                                "anonymized_name": "40e24f0c6b04b6d4657147990d6f39bd",
                                "parent_class": "StoreEvaluationParametersAction",
                            },
                            {
                                "anonymized_name": "2b99b6b280b8a6ad1176f37580a16411",
                                "parent_class": "UpdateDataDocsAction",
                            },
                        ],
                    },
                ],
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:28.070Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "checkpoint.run",
            "event_payload": {
                "anonymized_name": "f563d9aa1604e16099bb7dff7b203319",
                "config_version": 1.0,
                "anonymized_validations": [
                    {
                        "anonymized_batch_request": {
                            "anonymized_batch_request_required_top_level_properties": {
                                "anonymized_datasource_name": GETTING_STARTED_DATASOURCE_NAME,
                                "anonymized_data_connector_name": "d52d7bff3226a7f94dd3510c1040de78",
                                "anonymized_data_asset_name": "556e8c06239d09fc66f424eabb9ca491",
                            },
                            "batch_request_optional_top_level_keys": [
                                "runtime_parameters",
                            ],
                            "runtime_parameters_keys": ["path"],
                        },
                        "anonymized_expectation_suite_name": "6a04fc37da0d43a4c21429f6788d2cff",
                        "anonymized_action_list": [
                            {
                                "anonymized_name": "8e3e134cd0402c3970a02f40d2edfc26",
                                "parent_class": "StoreValidationResultAction",
                            },
                            {
                                "anonymized_name": "40e24f0c6b04b6d4657147990d6f39bd",
                                "parent_class": "StoreEvaluationParametersAction",
                            },
                            {
                                "anonymized_name": "2b99b6b280b8a6ad1176f37580a16411",
                                "parent_class": "UpdateDataDocsAction",
                            },
                        ],
                    },
                ],
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:28.070Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
    ],
    "data_context.run_validation_operator": data_context_run_validation_operator_events,
    "legacy_profiler.build_suite": [
        {
            "event": "legacy_profiler.build_suite",
            "event_payload": {
                "profile_dataset_type": "Dataset",
                "excluded_expectations_specified": False,
                "ignored_columns_specified": True,
                "not_null_only": False,
                "primary_or_compound_key_specified": True,
                "semantic_types_dict_specified": False,
                "table_expectations_only": False,
                "value_set_threshold_specified": True,
                "api_version": "v2",
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:28.070Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "legacy_profiler.build_suite",
            "event_payload": {
                "profile_dataset_type": "Batch",
                "excluded_expectations_specified": True,
                "ignored_columns_specified": False,
                "not_null_only": False,
                "primary_or_compound_key_specified": False,
                "semantic_types_dict_specified": False,
                "table_expectations_only": True,
                "value_set_threshold_specified": True,
                "api_version": "v2",
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:23.570Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "legacy_profiler.build_suite",
            "event_payload": {
                "profile_dataset_type": "Validator",
                "excluded_expectations_specified": False,
                "ignored_columns_specified": True,
                "not_null_only": True,
                "primary_or_compound_key_specified": False,
                "semantic_types_dict_specified": True,
                "table_expectations_only": False,
                "value_set_threshold_specified": False,
                "api_version": "v2",
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:23.570Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "legacy_profiler.build_suite",
            "event_payload": {
                "profile_dataset_type": "Validator",
                "excluded_expectations_specified": False,
                "ignored_columns_specified": False,
                "not_null_only": False,
                "primary_or_compound_key_specified": False,
                "semantic_types_dict_specified": False,
                "table_expectations_only": False,
                "value_set_threshold_specified": True,
                "api_version": "v2",
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:23.570Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "legacy_profiler.build_suite",
            "event_payload": {
                "profile_dataset_type": "Validator",
                "excluded_expectations_specified": True,
                "ignored_columns_specified": True,
                "not_null_only": False,
                "primary_or_compound_key_specified": True,
                "semantic_types_dict_specified": False,
                "table_expectations_only": False,
                "value_set_threshold_specified": True,
                "api_version": "v2",
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:23.570Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "legacy_profiler.build_suite",
            "event_payload": {
                "profile_dataset_type": "Validator",
                "excluded_expectations_specified": True,
                "ignored_columns_specified": True,
                "not_null_only": False,
                "primary_or_compound_key_specified": True,
                "semantic_types_dict_specified": True,
                "table_expectations_only": False,
                "value_set_threshold_specified": True,
                "api_version": "v2",
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:23.570Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "legacy_profiler.build_suite",
            "event_payload": {
                "profile_dataset_type": "Validator",
                "excluded_expectations_specified": True,
                "ignored_columns_specified": True,
                "not_null_only": False,
                "primary_or_compound_key_specified": False,
                "semantic_types_dict_specified": False,
                "table_expectations_only": True,
                "value_set_threshold_specified": True,
                "api_version": "v2",
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:23.570Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "legacy_profiler.build_suite",
            "event_payload": {
                "profile_dataset_type": "Validator",
                "excluded_expectations_specified": True,
                "ignored_columns_specified": True,
                "not_null_only": False,
                "primary_or_compound_key_specified": False,
                "semantic_types_dict_specified": False,
                "table_expectations_only": True,
                "value_set_threshold_specified": True,
                "api_version": "v2",
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:23.570Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.45.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
    ],
    "data_context.save_expectation_suite": [
        {
            "event": "data_context.save_expectation_suite",
            "event_payload": {
                "anonymized_expectation_suite_name": "4b6bf73298fcc2db6da929a8f18173f7"
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:23.570Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.11.9.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
    ],
    "profiler.result.get_expectation_suite": [
        {
            "event": "profiler.result.get_expectation_suite",
            "event_payload": {
                "anonymized_expectation_suite_name": "4b6bf73298fcc2db6da929a8f18173f7"
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:23.570Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.11.9.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
    ],
    "data_assistant.result.get_expectation_suite": [
        {
            "event": "data_assistant.result.get_expectation_suite",
            "event_payload": {
                "anonymized_expectation_suite_name": "4b6bf73298fcc2db6da929a8f18173f7"
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:23.570Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.11.9.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
    ],
    # BaseDataContext.test_yaml_config() MESSAGES
    "data_context.test_yaml_config": generate_messages_with_defaults(
        defaults={
            "success": True,
            "version": "1.0.0",
            "event_time": "2021-06-18T14:36:58.837Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
        },
        message_stubs=[
            {
                "event": "data_context.test_yaml_config",
                "event_payload": {
                    "anonymized_name": "fake_anonymized_name_for_testing",
                    "parent_class": class_name,
                    "diagnostic_info": [],
                },
                "ge_version": "0.13.20.manual_testing",
            }
            for class_name in _YamlConfigValidator.ALL_TEST_YAML_CONFIG_SUPPORTED_TYPES
        ]
        + [
            {
                "event": "data_context.test_yaml_config",
                "success": False,
                "event_payload": {
                    "anonymized_name": "fake_anonymized_name_for_testing",
                    "parent_class": class_name,
                    "diagnostic_info": [],
                },
                "ge_version": "0.13.20.manual_testing",
            }
            for class_name in _YamlConfigValidator.ALL_TEST_YAML_CONFIG_SUPPORTED_TYPES
        ]
        # Diagnostic Message Types
        + [
            {
                "event": "data_context.test_yaml_config",
                "success": False,
                "event_payload": {
                    "diagnostic_info": ["__substitution_error__"],
                },
                "ge_version": "0.13.20.manual_testing",
            },
            {
                "event": "data_context.test_yaml_config",
                "success": False,
                "event_payload": {
                    "diagnostic_info": ["__yaml_parse_error__"],
                },
                "ge_version": "0.13.20.manual_testing",
            },
            {
                "event": "data_context.test_yaml_config",
                "success": True,
                "event_payload": {
                    "diagnostic_info": ["__custom_subclass_not_core_ge__"],
                },
                "ge_version": "0.13.20.manual_testing",
            },
            {
                "event": "data_context.test_yaml_config",
                "success": True,
                "event_payload": {
                    "diagnostic_info": ["__class_name_not_provided__"],
                },
                "ge_version": "0.13.20.manual_testing",
            },
            {
                "event": "data_context.test_yaml_config",
                "success": False,
                "event_payload": {
                    "diagnostic_info": ["__class_name_not_provided__"],
                },
                "ge_version": "0.13.20.manual_testing",
            },
        ]
        # Store Message Types
        + [
            {
                "event": "data_context.test_yaml_config",
                "success": True,
                "event_payload": {
                    "anonymized_name": "fake_anonymized_name_for_testing",
                    "parent_class": "ExpectationsStore",
                    "anonymized_store_backend": {
                        "parent_class": "InMemoryStoreBackend"
                    },
                },
                "ge_version": "0.13.20.manual_testing",
            },
        ]
        # Datasource Message Types
        + [
            {
                "event": "data_context.test_yaml_config",
                "success": True,
                "event_payload": {
                    "anonymized_name": "fake_anonymized_name_for_testing",
                    "parent_class": "Datasource",
                    "anonymized_execution_engine": {
                        "anonymized_name": "fake_anonymized_name_for_testing",
                        "parent_class": "PandasExecutionEngine",
                    },
                    "anonymized_data_connectors": [
                        {
                            "anonymized_name": "fake_anonymized_name_for_testing",
                            "parent_class": "InferredAssetFilesystemDataConnector",
                        },
                    ],
                },
                "ge_version": "0.13.20.manual_testing",
            },
        ]
        # DataConnector Message Types
        + [
            {
                "event": "data_context.test_yaml_config",
                "success": True,
                "event_payload": {
                    "anonymized_name": "fake_anonymized_name_for_testing",
                    "parent_class": "ConfiguredAssetFilesystemDataConnector",
                },
                "ge_version": "0.13.20.manual_testing",
            },
        ]
        # Checkpoint Message Types
        + [
            {
                "event": "data_context.test_yaml_config",
                "success": True,
                "event_payload": {
                    "anonymized_name": "fake_anonymized_name_for_testing",
                    "parent_class": "Checkpoint",
                },
                "ge_version": "0.13.20.manual_testing",
            },
        ],
    ),
    "datasource.sqlalchemy.connect": [
        {
            "event": "datasource.sqlalchemy.connect",
            "event_payload": {
                "anonymized_name": "6989a7654d0e27470dc01292b6ed0dea",
                "sqlalchemy_dialect": "postgresql",
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T00:38:32.664Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.11.5.manual_testing",
        },
    ],
    "execution_engine.sqlalchemy.connect": [
        {
            "event": "execution_engine.sqlalchemy.connect",
            "event_payload": {
                "anonymized_name": "6989a7654d0e27470dc01292b6ed0dea",
                "sqlalchemy_dialect": "postgresql",
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T00:38:32.664Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.0.manual_testing",
        },
    ],
    "expectation_suite.add_expectation": [
        {
            "event_payload": {},
            "event": "expectation_suite.add_expectation",
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:28.070Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.47.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        }
    ],
    "data_context.run_profiler_with_dynamic_arguments": [
        {
            "event_payload": {},
            "event": "data_context.run_profiler_with_dynamic_arguments",
            "success": True,
            "version": "1.0.0",
            "event_time": "2022-04-21T16:08:28.070Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.14.6.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
    ],
    "data_context.run_profiler_on_data": [
        {
            "event_payload": {},
            "event": "data_context.run_profiler_on_data",
            "success": True,
            "version": "1.0.0",
            "event_time": "2022-04-21T16:08:28.070Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.14.8.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        }
    ],
    # CLI INIT COMMANDS
    "cli.init.create": [
        {
            "event": "cli.init.create",
            "event_payload": {},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:06:47.697Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.11.9.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "cli.init.create",
            "event_payload": {"api_version": "v2"},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:06:47.697Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.0.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "cli.init.create",
            "event_payload": {"api_version": "v3"},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:06:47.697Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.0.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
    ],
    # CLI PROJECT COMMANDS
    "cli.project.check_config": [
        {
            "event": "cli.project.check_config",
            "event_payload": {},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-03T23:42:34.068Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.11.9.manual_testing",
        },
        {
            "event": "cli.project.check_config",
            "event_payload": {"api_version": "v2"},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-03T23:42:34.068Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.0.manual_testing",
        },
        {
            "event": "cli.project.check_config",
            "event_payload": {"api_version": "v3"},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-03T23:42:34.068Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.0.manual_testing",
        },
    ],
    "cli.project.upgrade": generate_messages_with_defaults(
        defaults={
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T00:20:37.828Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
        },
        message_stubs=[
            {
                "event": "cli.project.upgrade.begin",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.project.upgrade.end",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.project.upgrade.end",
                "success": False,
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
        ],
    ),
    # CLI STORE COMMANDS
    "cli.store.list": generate_messages_with_defaults(
        defaults={
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-03T23:56:53.908Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
        },
        message_stubs=[
            {
                "event": "cli.store.list",
                "event_payload": {},
                "ge_version": "0.11.9.manual_testing",
            },
            {
                "event": "cli.store.list",
                "event_payload": {"api_version": "v2"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.store.list",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.store.list.begin",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.store.list.end",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.store.list.end",
                "success": False,
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
        ],
    ),
    # CLI DATASOURCE COMMANDS
    "cli.datasource.list": generate_messages_with_defaults(
        defaults={
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T22:50:58.837Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
        },
        message_stubs=[
            {
                "event": "cli.datasource.list",
                "event_payload": {},
                "ge_version": "0.11.9.manual_testing",
            },
            {
                "event": "cli.datasource.list",
                "event_payload": {"api_version": "v2"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.datasource.list",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.datasource.list.begin",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.datasource.list.end",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.datasource.list.end",
                "success": False,
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
        ],
    ),
    "cli.datasource.new": generate_messages_with_defaults(
        defaults={
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T22:50:58.837Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
        },
        message_stubs=[
            {
                "event": "cli.datasource.new",
                "event_payload": {},
                "ge_version": "0.11.9.manual_testing",
            },
            {
                "event": "cli.datasource.new",
                "event_payload": {"api_version": "v2"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.datasource.new",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.datasource.new.begin",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.datasource.new.end",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.datasource.new.end",
                "success": False,
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
        ],
    ),
    "cli.datasource.delete": generate_messages_with_defaults(
        defaults={
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T22:50:58.837Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
        },
        message_stubs=[
            {
                "event": "cli.datasource.delete",
                "event_payload": {},
                "ge_version": "0.11.9.manual_testing",
            },
            {
                "event": "cli.datasource.delete",
                "event_payload": {"api_version": "v2"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.datasource.delete",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.datasource.delete.begin",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.datasource.delete.end",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.datasource.delete.end",
                "success": False,
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.datasource.delete.end",
                "event_payload": {"api_version": "v3", "cancelled": True},
                "ge_version": "0.13.18.manual_testing",
            },
        ],
    ),
    "cli.datasource.profile": [
        {
            "event": "cli.datasource.profile",
            "event_payload": {},
            "success": False,
            "version": "1.0.0",
            "event_time": "2020-08-05T01:03:17.567Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.11.9.manual_testing",
        },
        {
            "event": "cli.datasource.profile",
            "event_payload": {"api_version": "v2"},
            "success": False,
            "version": "1.0.0",
            "event_time": "2020-08-05T01:03:17.567Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.0.manual_testing",
        },
        {
            "event": "cli.datasource.profile",
            "event_payload": {"api_version": "v3"},
            "success": False,
            "version": "1.0.0",
            "event_time": "2020-08-05T01:03:17.567Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.0.manual_testing",
        },
    ],
    # CLI NEW_DS_CHOICE COMMANDS
    "cli.new_ds_choice": [
        {
            "event": "cli.new_ds_choice",
            "event_payload": {"type": "pandas"},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:08.963Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.11.9.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "cli.new_ds_choice",
            "event_payload": {"type": "pandas", "api_version": "v2"},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:08.963Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.0.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "cli.new_ds_choice",
            "event_payload": {"type": "pandas", "api_version": "v3"},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:08.963Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.0.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
    ],
    # CLI SUITE COMMANDS
    "cli.suite.demo": generate_messages_with_defaults(
        defaults={
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T22:50:58.837Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
        },
        message_stubs=[
            {
                "event": "cli.suite.demo",
                "event_payload": {},
                "ge_version": "0.11.9.manual_testing",
            },
            {
                "event": "cli.suite.demo",
                "event_payload": {"api_version": "v2"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.suite.demo",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.suite.demo.begin",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.suite.demo.end",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.suite.demo.end",
                "success": False,
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
        ],
    ),
    "cli.suite.list": generate_messages_with_defaults(
        defaults={
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T22:50:58.837Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
        },
        message_stubs=[
            {
                "event": "cli.suite.list",
                "event_payload": {},
                "ge_version": "0.11.9.manual_testing",
            },
            {
                "event": "cli.suite.list",
                "event_payload": {"api_version": "v2"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.suite.list",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.suite.list.begin",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.suite.list.end",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.suite.list.end",
                "success": False,
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
        ],
    ),
    "cli.suite.new": generate_messages_with_defaults(
        defaults={
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T22:50:58.837Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
        },
        message_stubs=[
            {
                "event": "cli.suite.new",
                "event_payload": {
                    "anonymized_expectation_suite_name": "0604e6a8f5a1da77e0438aa3b543846e"
                },
                "ge_version": "0.11.9.manual_testing",
            },
            {
                "event": "cli.suite.new",
                "event_payload": {
                    "anonymized_expectation_suite_name": "0604e6a8f5a1da77e0438aa3b543846e",
                    "api_version": "v2",
                },
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.suite.new",
                "event_payload": {
                    "anonymized_expectation_suite_name": "0604e6a8f5a1da77e0438aa3b543846e",
                    "api_version": "v3",
                },
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.suite.new.begin",
                "event_payload": {
                    "anonymized_expectation_suite_name": "0604e6a8f5a1da77e0438aa3b543846e",
                    "api_version": "v3",
                },
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.suite.new.end",
                "event_payload": {
                    "anonymized_expectation_suite_name": "0604e6a8f5a1da77e0438aa3b543846e",
                    "api_version": "v3",
                },
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.suite.new.end",
                "success": False,
                "event_payload": {
                    "anonymized_expectation_suite_name": "0604e6a8f5a1da77e0438aa3b543846e",
                    "api_version": "v3",
                },
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.suite.new.end",
                "event_payload": {
                    "anonymized_expectation_suite_name": "0604e6a8f5a1da77e0438aa3b543846e",
                    **CLISuiteInteractiveFlagCombinations.PROMPTED_CHOICE_TRUE_PROFILE_TRUE.value,
                    "api_version": "v3",
                },
                "ge_version": "0.13.18.manual_testing",
            },
        ],
    ),
    "cli.suite.edit": generate_messages_with_defaults(
        defaults={
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T22:50:58.837Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
        },
        message_stubs=[
            {
                "event": "cli.suite.edit",
                "event_payload": {
                    "anonymized_expectation_suite_name": "0604e6a8f5a1da77e0438aa3b543846e"
                },
                "ge_version": "0.11.9.manual_testing",
            },
            {
                "event": "cli.suite.edit",
                "event_payload": {
                    "anonymized_expectation_suite_name": "0604e6a8f5a1da77e0438aa3b543846e",
                    "api_version": "v2",
                },
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.suite.edit",
                "event_payload": {
                    "anonymized_expectation_suite_name": "0604e6a8f5a1da77e0438aa3b543846e",
                    "api_version": "v3",
                },
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.suite.edit.begin",
                "event_payload": {
                    "anonymized_expectation_suite_name": "0604e6a8f5a1da77e0438aa3b543846e",
                    "api_version": "v3",
                },
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.suite.edit.end",
                "event_payload": {
                    "anonymized_expectation_suite_name": "0604e6a8f5a1da77e0438aa3b543846e",
                    "api_version": "v3",
                },
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.suite.edit.end",
                "success": False,
                "event_payload": {
                    "anonymized_expectation_suite_name": "0604e6a8f5a1da77e0438aa3b543846e",
                    "api_version": "v3",
                },
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.suite.edit.end",
                "event_payload": {
                    "anonymized_expectation_suite_name": "0604e6a8f5a1da77e0438aa3b543846e",
                    **CLISuiteInteractiveFlagCombinations.PROMPTED_CHOICE_DEFAULT.value,
                    "api_version": "v3",
                },
                "ge_version": "0.13.18.manual_testing",
            },
        ],
    ),
    "cli.suite.delete": generate_messages_with_defaults(
        defaults={
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T22:50:58.837Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
        },
        message_stubs=[
            {
                "event": "cli.suite.delete",
                "event_payload": {},
                "ge_version": "0.11.9.manual_testing",
            },
            {
                "event": "cli.suite.delete",
                "event_payload": {"api_version": "v2"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.suite.delete",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.suite.delete.begin",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.suite.delete.end",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.suite.delete.end",
                "success": False,
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.suite.delete.end",
                "event_payload": {"api_version": "v3", "cancelled": True},
                "ge_version": "0.13.18.manual_testing",
            },
        ],
    ),
    "cli.suite.scaffold": [
        {
            "event": "cli.suite.scaffold",
            "event_payload": {},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-05T00:58:51.961Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.11.9.manual_testing",
        },
        {
            "event": "cli.suite.scaffold",
            "event_payload": {"api_version": "v2"},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-05T00:58:51.961Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.0.manual_testing",
        },
        {
            "event": "cli.suite.scaffold",
            "event_payload": {"api_version": "v3"},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-05T00:58:51.961Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.0.manual_testing",
        },
    ],
    # CLI CHECKPOINT COMMANDS
    "cli.checkpoint.new": generate_messages_with_defaults(
        defaults={
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T22:50:58.837Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
        },
        message_stubs=[
            {
                "event": "cli.checkpoint.new",
                "event_payload": {},
                "ge_version": "0.11.9.manual_testing",
            },
            {
                "event": "cli.checkpoint.new",
                "event_payload": {"api_version": "v2"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.checkpoint.new",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.checkpoint.new.begin",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.checkpoint.new.end",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.checkpoint.new.end",
                "success": False,
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
        ],
    ),
    "cli.checkpoint.script": generate_messages_with_defaults(
        defaults={
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T22:50:58.837Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
        },
        message_stubs=[
            {
                "event": "cli.checkpoint.script",
                "event_payload": {},
                "ge_version": "0.11.9.manual_testing",
            },
            {
                "event": "cli.checkpoint.script",
                "event_payload": {"api_version": "v2"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.checkpoint.script",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.checkpoint.script.begin",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.checkpoint.script.end",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.checkpoint.script.end",
                "success": False,
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
        ],
    ),
    "cli.checkpoint.run": generate_messages_with_defaults(
        defaults={
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T22:50:58.837Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
        },
        message_stubs=[
            {
                "event": "cli.checkpoint.run",
                "event_payload": {},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.checkpoint.run",
                "event_payload": {"api_version": "v2"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.checkpoint.run",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.checkpoint.run.begin",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.checkpoint.run.end",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.checkpoint.run.end",
                "success": False,
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
        ],
    ),
    "cli.checkpoint.list": generate_messages_with_defaults(
        defaults={
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T22:50:58.837Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
        },
        message_stubs=[
            {
                "event": "cli.checkpoint.list",
                "event_payload": {},
                "ge_version": "0.11.9.manual_testing",
            },
            {
                "event": "cli.checkpoint.list",
                "event_payload": {"api_version": "v2"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.checkpoint.list",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.checkpoint.list.begin",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.checkpoint.list.end",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.checkpoint.list.end",
                "success": False,
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
        ],
    ),
    "cli.checkpoint.delete": generate_messages_with_defaults(
        defaults={
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T22:50:58.837Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
        },
        message_stubs=[
            {
                "event": "cli.checkpoint.delete",
                "event_payload": {},
                "ge_version": "0.11.9.manual_testing",
            },
            {
                "event": "cli.checkpoint.delete",
                "event_payload": {"api_version": "v2"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.checkpoint.delete",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.checkpoint.delete.begin",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.checkpoint.delete.end",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.checkpoint.delete.end",
                "success": False,
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.checkpoint.delete.end",
                "event_payload": {"api_version": "v3", "cancelled": True},
                "ge_version": "0.13.18.manual_testing",
            },
        ],
    ),
    # CLI VALIDATION_OPERATOR COMMANDS
    "cli.validation_operator.list": [
        {
            "event": "cli.validation_operator.list",
            "event_payload": {},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-03T23:32:33.635Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.11.9.manual_testing",
        },
        {
            "event": "cli.validation_operator.list",
            "event_payload": {"api_version": "v2"},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-03T23:32:33.635Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.0.manual_testing",
        },
    ],
    "cli.validation_operator.run": [
        {
            "event": "cli.validation_operator.run",
            "event_payload": {},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-03T23:33:15.664Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.11.9.manual_testing",
        },
        {
            "event": "cli.validation_operator.run",
            "event_payload": {"api_version": "v2"},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-03T23:33:15.664Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.13.0.manual_testing",
        },
    ],
    # CLI DOCS COMMANDS
    "cli.docs.build": generate_messages_with_defaults(
        defaults={
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T00:25:27.088Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
        },
        message_stubs=[
            {
                "event": "cli.docs.build",
                "event_payload": {},
                "ge_version": "0.11.9.manual_testing",
            },
            {
                "event": "cli.docs.build",
                "event_payload": {"api_version": "v2"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.docs.build",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.docs.build.begin",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.docs.build.end",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.docs.build.end",
                "success": False,
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.docs.build.end",
                "event_payload": {"api_version": "v3", "cancelled": True},
                "ge_version": "0.13.18.manual_testing",
            },
        ],
    ),
    "cli.docs.clean": generate_messages_with_defaults(
        defaults={
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-05T00:36:50.979Z",
            "data_context_id": "2a948908-ec42-47f2-b972-c07bb0393de4",
            "data_context_instance_id": "e7e0916d-d527-437a-b89d-5eb8c36d408f",
        },
        message_stubs=[
            {
                "event": "cli.docs.clean",
                "event_payload": {},
                "ge_version": "0.11.9+25.g3ca555c.dirty",
            },
            {
                "event": "cli.docs.clean",
                "event_payload": {"api_version": "v2"},
                "ge_version": "0.13.0+25.g3ca555c.dirty",
            },
            {
                "event": "cli.docs.clean",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.0+25.g3ca555c.dirty",
            },
            {
                "event": "cli.docs.clean.begin",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.docs.clean.end",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.docs.clean.end",
                "success": False,
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.docs.clean.end",
                "event_payload": {"api_version": "v3", "cancelled": True},
                "ge_version": "0.13.18.manual_testing",
            },
        ],
    ),
    "cli.docs.list": generate_messages_with_defaults(
        defaults={
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-08-04T00:20:37.828Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
        },
        message_stubs=[
            {
                "event": "cli.docs.list",
                "event_payload": {},
                "ge_version": "0.11.9.manual_testing",
            },
            {
                "event": "cli.docs.list",
                "event_payload": {"api_version": "v2"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.docs.list",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.0.manual_testing",
            },
            {
                "event": "cli.docs.list.begin",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.docs.list.end",
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
            {
                "event": "cli.docs.list.end",
                "success": False,
                "event_payload": {"api_version": "v3"},
                "ge_version": "0.13.18.manual_testing",
            },
        ],
    ),
    "profiler.run": [
        {
            "event": "profiler.run",
            "event_payload": {},
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:28.070Z",
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.14.7.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "profiler.run",
            "event_payload": {
                "anonymized_name": "5b6c98e19e21e77191fb071bb9e80070",
                "anonymized_rules": [
                    {
                        "anonymized_domain_builder": {
                            "parent_class": "TableDomainBuilder"
                        },
                        "anonymized_expectation_configuration_builders": [
                            {
                                "parent_class": "DefaultExpectationConfigurationBuilder",
                                "expectation_type": "expect_column_pair_values_A_to_be_greater_than_B",
                            }
                        ],
                        "anonymized_name": "5a83f3728393d6519a197cffdccd50ff",
                        "anonymized_parameter_builders": [
                            {
                                "anonymized_name": "9349ed253aba01f4ecf190af61018a11",
                                "anonymized_class": "dasfj238fefasfa90sdf23j39202f2j2",
                                "parent_class": "MetricMultiBatchParameterBuilder",
                            }
                        ],
                    },
                    {
                        "anonymized_domain_builder": {
                            "parent_class": "TableDomainBuilder"
                        },
                        "anonymized_expectation_configuration_builders": [
                            {
                                "parent_class": "DefaultExpectationConfigurationBuilder",
                                "anonymized_expectation_type": "239asdfjaew832hg20sdjwd9922e9e2u",
                            }
                        ],
                        "anonymized_name": "0bac2cecbb0cf8bb704e86710941434e",
                        "anonymized_parameter_builders": [
                            {
                                "anonymized_name": "b7719efec76c6ebe30230fc1ec023beb",
                                "parent_class": "MetricMultiBatchParameterBuilder",
                            }
                        ],
                    },
                ],
                "config_version": 1.0,
                "rule_count": 2,
                "variable_count": 1,
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:28.070Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.14.7.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
        {
            "event": "profiler.run",
            "event_payload": {
                "anonymized_name": "5b6c98e19e21e77191fb071bb9e80070",
                "anonymized_rules": [
                    {
                        "anonymized_domain_builder": {
                            "anonymized_class": "d2972bccf7a2a0ff91ba9369a86dcbe1",
                            "parent_class": "TableDomainBuilder",
                        },
                        "anonymized_expectation_configuration_builders": [
                            {
                                "anonymized_class": "0d70a2037f19cf1764afad97c7395167",
                                "anonymized_expectation_type": "c7c23fbf56041786bf024a2407031b27",
                                "parent_class": "DefaultExpectationConfigurationBuilder",
                            }
                        ],
                        "anonymized_name": "5a83f3728393d6519a197cffdccd50ff",
                        "anonymized_parameter_builders": [
                            {
                                "anonymized_class": "c73849d7016ce7ab68e24465361a717a",
                                "anonymized_name": "9349ed253aba01f4ecf190af61018a11",
                                "parent_class": "RegexPatternStringParameterBuilder",
                            }
                        ],
                    },
                    {
                        "anonymized_domain_builder": {
                            "anonymized_class": "df79fd715bf3ea514c3f4e3006025b24",
                            "parent_class": "__not_recognized__",
                        },
                        "anonymized_expectation_configuration_builders": [
                            {
                                "anonymized_class": "71128204dee66972b5cfc8851b216508",
                                "anonymized_expectation_type": "828b498d29af626836697ba1622ca234",
                                "parent_class": "__not_recognized__",
                            }
                        ],
                        "anonymized_name": "0bac2cecbb0cf8bb704e86710941434e",
                        "anonymized_parameter_builders": [
                            {
                                "anonymized_class": "fcbd493bd096d894cf83506bc23a0729",
                                "anonymized_name": "5af4c6b6dedc5f9b3b840709e957c4ed",
                                "parent_class": "__not_recognized__",
                            }
                        ],
                    },
                ],
                "config_version": 1.0,
                "rule_count": 2,
                "variable_count": 1,
            },
            "success": True,
            "version": "1.0.0",
            "event_time": "2020-06-25T16:08:28.070Z",
            "event_duration": 123,
            "data_context_id": "00000000-0000-0000-0000-000000000002",
            "data_context_instance_id": "10000000-0000-0000-0000-000000000002",
            "ge_version": "0.14.7.manual_testing",
            "x-forwarded-for": "00.000.00.000, 00.000.000.000",
        },
    ],
    UsageStatsEvents.CLOUD_MIGRATE: [cloud_migrate],
}

test_messages = []
message_test_ids = []
for message_type, messages in valid_usage_statistics_messages.items():
    for idx, test_message in enumerate(messages):
        test_messages += [test_message]
        message_test_ids += [f"{message_type}_{idx}"]


@pytest.mark.aws_integration
@pytest.mark.parametrize("message", test_messages, ids=message_test_ids)
def test_usage_statistics_message(
    message: dict, requests_session_with_retries: requests.Session
):
    """known message formats should be valid"""
    res = requests_session_with_retries.post(
        USAGE_STATISTICS_QA_URL, json=message, timeout=2
    )
    assert res.status_code == 201
    assert res.json() == {"event_count": 1}



import os

from ruamel.yaml import YAML

import great_expectations as gx
from great_expectations.data_context.types.base import (
    DataContextConfig,
    DataContextConfigSchema,
)
from great_expectations.data_context.util import file_relative_path
from tests.data_context.conftest import create_data_context_files

yaml = YAML()
yaml.indent(mapping=2, sequence=4, offset=2)
yaml.default_flow_style = False

dataContextConfigSchema = DataContextConfigSchema()


def test_substituted_config_variables_not_written_to_file(tmp_path_factory):
    # this test uses a great_expectations.yml with almost all values replaced
    # with substitution variables

    project_path = str(tmp_path_factory.mktemp("data_context"))
    context_path = os.path.join(project_path, "great_expectations")
    asset_config_path = os.path.join(context_path, "expectations")

    create_data_context_files(
        context_path,
        asset_config_path,
        ge_config_fixture_filename="great_expectations_basic_with_exhaustive_variables.yml",
        config_variables_fixture_filename="config_variables_exhaustive.yml",
    )

    # load ge config fixture for expected
    path_to_yml = (
        "../test_fixtures/great_expectations_basic_with_exhaustive_variables.yml"
    )
    path_to_yml = file_relative_path(__file__, path_to_yml)
    with open(path_to_yml) as data:
        config_commented_map_from_yaml = yaml.load(data)
    expected_config = DataContextConfig.from_commented_map(
        config_commented_map_from_yaml
    )
    expected_config_commented_map = dataContextConfigSchema.dump(expected_config)
    expected_config_commented_map.pop("anonymous_usage_statistics")

    # instantiate data_context twice to go through cycle of loading config from file then saving
    context = gx.get_context(context_root_dir=context_path)
    context._save_project_config()
    context_config_commented_map = dataContextConfigSchema.dump(
        gx.get_context(context_root_dir=context_path)._project_config
    )
    context_config_commented_map.pop("anonymous_usage_statistics")

    assert context_config_commented_map == expected_config_commented_map



# TODO: ADD TESTS ONCE GET_BATCH IS INTEGRATED!

import dateutil.parser
import pandas as pd
import pytest
from freezegun import freeze_time

import great_expectations as gx
from great_expectations.core.run_identifier import RunIdentifier
from great_expectations.self_check.util import modify_locale
from great_expectations.util import get_context
from great_expectations.validation_operators.validation_operators import (
    WarningAndFailureExpectationSuitesValidationOperator,
)


@pytest.fixture
def assets_to_validate():
    # succeeded "failure-level" suite
    # failed "warning-level" suite
    my_df_1 = pd.DataFrame({"x": [1, 2, 3, 4, 5], "y": [1, 2, 3, 4, None]})
    my_ge_df_1 = gx.dataset.PandasDataset(
        my_df_1, batch_kwargs={"ge_batch_id": "82a8de83-e063-11e9-8226-acde48001122"}
    )

    # failed "failure-level" suite
    # failed "warning-level" suite
    my_df_2 = pd.DataFrame({"x": [1, 2, 3, 4, 99], "y": [1, 2, 3, 4, None]})
    my_ge_df_2 = gx.dataset.PandasDataset(
        my_df_2, batch_kwargs={"ge_batch_id": "82a8de83-e063-11e9-8133-acde48001122"}
    )

    # succeeded "failure-level" suite
    # succeeded "warning-level" suite
    my_df_3 = pd.DataFrame({"x": [1, 2, 3, 4, 5], "y": [1, 2, 3, 4, 5]})
    my_ge_df_3 = gx.dataset.PandasDataset(
        my_df_3, batch_kwargs={"ge_batch_id": "82a8de83-e063-11e9-a53d-acde48001122"}
    )

    # failed "failure-level" suite
    # succeeded "warning-level" suite
    my_df_4 = pd.DataFrame({"x": [1, 2, 3, 4, 99], "y": [1, 2, 3, 4, 5]})
    my_ge_df_4 = gx.dataset.PandasDataset(
        my_df_4, batch_kwargs={"ge_batch_id": "82a8de83-e063-11e9-8133-acde48001122"}
    )
    return [my_ge_df_1, my_ge_df_2, my_ge_df_3, my_ge_df_4]


@pytest.fixture
def warning_failure_validation_operator_data_context(
    basic_data_context_config_for_validation_operator,
    tmp_path_factory,
    filesystem_csv_4,
):
    project_path = str(tmp_path_factory.mktemp("great_expectations"))

    # NOTE: This setup is almost identical to test_DefaultDataContextAwareValidationOperator.
    # Consider converting to a single fixture.

    data_context = get_context(
        basic_data_context_config_for_validation_operator,
        project_path,
    )

    data_context.add_datasource(
        "my_datasource",
        class_name="PandasDatasource",
        batch_kwargs_generators={
            "subdir_reader": {
                "class_name": "SubdirReaderBatchKwargsGenerator",
                "base_directory": str(filesystem_csv_4),
            }
        },
    )

    data_context.add_expectation_suite(expectation_suite_name="f1.failure")
    df = data_context.get_batch(
        expectation_suite_name="f1.failure",
        batch_kwargs=data_context.build_batch_kwargs(
            "my_datasource", "subdir_reader", "f1"
        ),
    )
    df.expect_column_values_to_be_between(column="x", min_value=1, max_value=9)
    failure_expectations = df.get_expectation_suite(discard_failed_expectations=False)

    failure_expectations.expectation_suite_name = "f1.failure"
    data_context.update_expectation_suite(expectation_suite=failure_expectations)

    data_context.add_expectation_suite(expectation_suite_name="f1.warning")
    df = data_context.get_batch(
        expectation_suite_name="f1.warning",
        batch_kwargs=data_context.build_batch_kwargs(
            "my_datasource", "subdir_reader", "f1"
        ),
    )
    df.expect_column_values_to_be_between(
        column="x", min_value=1, max_value=9, mostly=0.8
    )
    df.expect_column_values_to_not_be_null(column="y")
    warning_expectations = df.get_expectation_suite(discard_failed_expectations=False)

    warning_expectations.expectation_suite_name = "f1.warning"
    data_context.update_expectation_suite(expectation_suite=warning_expectations)

    failure_expectations.expectation_suite_name = "f2.failure"
    data_context.add_expectation_suite(expectation_suite=failure_expectations)

    failure_expectations.expectation_suite_name = "f3.failure"
    data_context.add_expectation_suite(expectation_suite=failure_expectations)

    warning_expectations.expectation_suite_name = "f2.warning"
    data_context.add_expectation_suite(expectation_suite=warning_expectations)

    warning_expectations.expectation_suite_name = "f3.warning"
    data_context.add_expectation_suite(expectation_suite=warning_expectations)

    return data_context


@modify_locale
@freeze_time("09/26/2019 13:42:41")
@pytest.mark.filterwarnings(
    "ignore:System time is way off:urllib3.exceptions.SystemTimeWarning:urllib3"
)
def test_errors_warnings_validation_operator_run_slack_query(
    warning_failure_validation_operator_data_context, assets_to_validate
):
    data_context = warning_failure_validation_operator_data_context

    vo = WarningAndFailureExpectationSuitesValidationOperator(
        data_context=data_context,
        action_list=[],
        name="test",
        slack_webhook="https://hooks.slack.com/services/test/slack/webhook",
    )

    return_obj = vo.run(
        assets_to_validate=assets_to_validate,
        run_id=RunIdentifier(run_name="test_100"),
        base_expectation_suite_name="f1",
    )
    slack_query = vo._build_slack_query(return_obj)
    expected_slack_query = {
        "blocks": [
            {"type": "divider"},
            {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": "*FailureVsWarning Validation Operator Completed.*",
                },
            },
            {"type": "divider"},
            {
                "type": "section",
                "text": {"type": "mrkdwn", "text": "*Status*: Failed :x:"},
            },
            {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": "*Batch Id List:* ['ge_batch_id=82a8de83-e063-11e9-8133-acde48001122', "
                    "'ge_batch_id=82a8de83-e063-11e9-8226-acde48001122', "
                    "'ge_batch_id=82a8de83-e063-11e9-a53d-acde48001122']",
                },
            },
            {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": "*Failed Batches:* ['f1.failure-ge_batch_id=82a8de83-e063-11e9-8133-acde48001122']",
                },
            },
            {
                "type": "section",
                "text": {"type": "mrkdwn", "text": "*Run Name:* test_100"},
            },
            {
                "type": "section",
                "text": {"type": "mrkdwn", "text": "*Run Time:* LOCALEDATE"},
            },
            {"type": "divider"},
            {
                "type": "context",
                "elements": [
                    {
                        "type": "mrkdwn",
                        "text": "Learn about FailureVsWarning Validation Operators at https://docs.greatexpectations.i"
                        "o/en/latest/reference/validation_operators/warning_and_failure_expectation_suites_val"
                        "idation_operator.html",
                    }
                ],
            },
        ]
    }

    # We're okay with system variation in locales (OS X likes 24 hour, but not Travis)
    slack_query["blocks"][7]["text"]["text"] = slack_query["blocks"][7]["text"][
        "text"
    ].replace("09/26/2019 13:42:41", "LOCALEDATE")
    slack_query["blocks"][7]["text"]["text"] = slack_query["blocks"][7]["text"][
        "text"
    ].replace("09/26/2019 01:42:41 PM", "LOCALEDATE")
    expected_slack_query["blocks"][7]["text"]["text"] = expected_slack_query["blocks"][
        7
    ]["text"]["text"].replace("09/26/2019 13:42:41", "LOCALEDATE")
    expected_slack_query["blocks"][7]["text"]["text"] = expected_slack_query["blocks"][
        7
    ]["text"]["text"].replace("09/26/2019 01:42:41 PM", "LOCALEDATE")

    import json

    print(json.dumps(slack_query, indent=2))
    print(json.dumps(expected_slack_query, indent=2))
    assert slack_query == expected_slack_query


def test_errors_warnings_validation_operator_failed_vo_result(
    warning_failure_validation_operator_data_context, assets_to_validate
):
    # this tests whether the WarningAndFailureExpectationSuitesValidationOperator properly returns
    # a failed ValidationOperatorResult if there is a failed validation with a suite severity level of "failure"

    data_context = warning_failure_validation_operator_data_context

    vo = WarningAndFailureExpectationSuitesValidationOperator(
        data_context=data_context,
        action_list=[],
        name="test",
    )

    # only pass asset that yields failed "failure-level" suite and succeeded "warning-level" suite
    return_obj = vo.run(
        assets_to_validate=[assets_to_validate[3]],
        run_id=RunIdentifier(run_name="test_100"),
        base_expectation_suite_name="f1",
    )
    run_results = list(return_obj.run_results.values())

    # make sure there is at least one failed validation with a "failure-level" suite
    assert any(
        [
            run_result
            for run_result in run_results
            if run_result["expectation_suite_severity_level"] == "failure"
            and not run_result["validation_result"].success
        ]
    )
    # no failed warning suites
    assert not any(
        [
            run_result
            for run_result in run_results
            if run_result["expectation_suite_severity_level"] == "warning"
            and not run_result["validation_result"].success
        ]
    )
    assert not return_obj.success

    # only pass asset that yields failed "failure-level" suite and failed "warning-level" suite
    return_obj_2 = vo.run(
        assets_to_validate=[assets_to_validate[1]],
        run_id=RunIdentifier(run_name="test_100"),
        base_expectation_suite_name="f1",
    )
    run_results_2 = list(return_obj_2.run_results.values())

    # make sure there is at least one failed validation with a "failure-level" suite
    assert any(
        [
            run_result
            for run_result in run_results_2
            if run_result["expectation_suite_severity_level"] == "failure"
            and not run_result["validation_result"].success
        ]
    )
    # with at least one failed warning suite
    assert any(
        [
            run_result
            for run_result in run_results_2
            if run_result["expectation_suite_severity_level"] == "warning"
            and not run_result["validation_result"].success
        ]
    )
    assert not return_obj_2.success


def test_errors_warnings_validation_operator_succeeded_vo_result_with_only_failed_warning_suite(
    warning_failure_validation_operator_data_context, assets_to_validate
):
    # this tests whether the WarningAndFailureExpectationSuitesValidationOperator properly returns
    # a failed ValidationOperatorResult if there is a failed validation with a suite severity level of "failure"

    data_context = warning_failure_validation_operator_data_context

    vo = WarningAndFailureExpectationSuitesValidationOperator(
        data_context=data_context,
        action_list=[],
        name="test",
    )

    # only pass asset that yields succeeded "failure-level" suite and failed "warning-level" suite
    return_obj = vo.run(
        assets_to_validate=[assets_to_validate[0]],
        run_id=RunIdentifier(run_name="test_100"),
        base_expectation_suite_name="f1",
    )
    run_results = list(return_obj.run_results.values())

    # make sure there are no failed validations with suite severity of failure
    assert not any(
        [
            run_result
            for run_result in run_results
            if run_result["expectation_suite_severity_level"] == "failure"
            and not run_result["validation_result"].success
        ]
    )
    # make sure there is at least one failed validation with suite severity of warning
    assert any(
        [
            run_result
            for run_result in run_results
            if run_result["expectation_suite_severity_level"] == "warning"
            and not run_result["validation_result"].success
        ]
    )
    assert return_obj.success

    # only pass asset that yields succeeded "failure-level" suite and succeeded "warning-level" suite
    return_obj_2 = vo.run(
        assets_to_validate=[assets_to_validate[2]],
        run_id=RunIdentifier(run_name="test_100"),
        base_expectation_suite_name="f1",
    )
    run_results_2 = list(return_obj_2.run_results.values())

    # make sure there are no failed validations with suite severity of failure
    assert not any(
        [
            run_result
            for run_result in run_results_2
            if run_result["expectation_suite_severity_level"] == "failure"
            and not run_result["validation_result"].success
        ]
    )
    # make sure there are no failed validation with suite severity of warning
    assert not any(
        [
            run_result
            for run_result in run_results_2
            if run_result["expectation_suite_severity_level"] == "warning"
            and not run_result["validation_result"].success
        ]
    )
    assert return_obj_2.success


def test_passing_run_id_as_a_parameter_to_warning_and_failure_vo(
    warning_failure_validation_operator_data_context, assets_to_validate
):
    # this tests whether the run_id, run_name and run_time passed to WarningAndFailureExpectationSuitesValidationOperator
    # is saved in the validation result.

    data_context = warning_failure_validation_operator_data_context

    vo = WarningAndFailureExpectationSuitesValidationOperator(
        data_context=data_context,
        action_list=[],
        name="test",
    )

    # pass run id
    user_run_name = "test_run_name"
    run_dt = dateutil.parser.parse("2021-09-11 1:47:03+00:00")

    return_obj = vo.run(
        assets_to_validate=[assets_to_validate[3]],
        run_id={"run_name": user_run_name, "run_time": run_dt},
        base_expectation_suite_name="f1",
    )
    run_results = list(return_obj.run_results.values())

    assert (
        run_results[0]["validation_result"]["meta"]["run_id"].run_name == user_run_name
    )
    assert run_results[0]["validation_result"]["meta"]["run_id"].run_time == run_dt
    assert (
        run_results[1]["validation_result"]["meta"]["run_id"].run_name == user_run_name
    )
    assert run_results[1]["validation_result"]["meta"]["run_id"].run_time == run_dt

    # pass run name
    return_obj = vo.run(
        assets_to_validate=[assets_to_validate[3]],
        run_name=user_run_name,
        base_expectation_suite_name="f1",
    )
    run_results = list(return_obj.run_results.values())

    assert (
        run_results[0]["validation_result"]["meta"]["run_id"].run_name == user_run_name
    )
    assert (
        run_results[1]["validation_result"]["meta"]["run_id"].run_name == user_run_name
    )

    # pass run time
    return_obj = vo.run(
        assets_to_validate=[assets_to_validate[3]],
        run_time=run_dt,
        base_expectation_suite_name="f1",
    )
    run_results = list(return_obj.run_results.values())

    assert run_results[0]["validation_result"]["meta"]["run_id"].run_time == run_dt
    assert run_results[1]["validation_result"]["meta"]["run_id"].run_time == run_dt



"""TODO: Add docstring"""
from __future__ import annotations

from typing import TYPE_CHECKING, Dict, List, cast

from marshmallow import Schema, fields, post_dump

from great_expectations.core.expectation_suite import (
    ExpectationSuite,
    ExpectationSuiteSchema,
)
from great_expectations.core.expectation_validation_result import (
    ExpectationSuiteValidationResult,
    ExpectationSuiteValidationResultSchema,
)
from great_expectations.core.util import convert_to_json_serializable
from great_expectations.data_context.data_context_variables import (
    DataContextVariables,  # noqa: TCH001
)
from great_expectations.data_context.types.base import (
    CheckpointConfig,
    CheckpointConfigSchema,
    DataContextConfigSchema,
    DatasourceConfig,
    DatasourceConfigSchema,
)
from great_expectations.rule_based_profiler.config.base import (
    RuleBasedProfilerConfig,
    RuleBasedProfilerConfigSchema,
    ruleBasedProfilerConfigSchema,
)

if TYPE_CHECKING:
    from great_expectations.data_context.data_context.abstract_data_context import (
        AbstractDataContext,
    )


class ConfigurationBundle:
    def __init__(self, context: AbstractDataContext) -> None:

        self._context = context
        self._context_id = context.data_context_id

        self._data_context_variables: DataContextVariables = context.variables

        self._datasources: List[DatasourceConfig] = self._get_all_datasources()
        self._expectation_suites: List[
            ExpectationSuite
        ] = self._get_all_expectation_suites()
        self._checkpoints: List[CheckpointConfig] = self._get_all_checkpoints()
        self._profilers: List[RuleBasedProfilerConfig] = self._get_all_profilers()

        # Treated slightly differently as we require the keys downstream when printing migration status.
        self._validation_results: Dict[
            str, ExpectationSuiteValidationResult
        ] = self._get_all_validation_results()

    @property
    def data_context_id(self) -> str:
        return self._context_id

    def is_usage_stats_enabled(self) -> bool:
        """Determine whether usage stats are enabled.

        Also returns false if there are no usage stats settings provided.

        Returns: Boolean of whether the usage statistics are enabled.

        """
        if self._data_context_variables.anonymous_usage_statistics:
            return self._data_context_variables.anonymous_usage_statistics.enabled
        else:
            return False

    @property
    def data_context_variables(self) -> DataContextVariables:
        return self._data_context_variables

    @property
    def datasources(self) -> List[DatasourceConfig]:
        return self._datasources

    @property
    def expectation_suites(self) -> List[ExpectationSuite]:
        return self._expectation_suites

    @property
    def checkpoints(self) -> List[CheckpointConfig]:
        return self._checkpoints

    @property
    def profilers(self) -> List[RuleBasedProfilerConfig]:
        return self._profilers

    @property
    def validation_results(self) -> Dict[str, ExpectationSuiteValidationResult]:
        return self._validation_results

    def _get_all_datasources(self) -> List[DatasourceConfig]:

        datasource_names: List[str] = list(self._context.datasources.keys())

        # Note: we are accessing the protected _datasource_store to not add a public property
        # to all Data Contexts.
        datasource_configs: List[DatasourceConfig] = []
        for datasource_name in datasource_names:
            datasource_config = self._context._datasource_store.retrieve_by_name(  # type: ignore[attr-defined]
                datasource_name=datasource_name
            )
            datasource_config.name = datasource_name
            datasource_configs.append(datasource_config)

        return datasource_configs

    def _get_all_expectation_suites(self) -> List[ExpectationSuite]:
        return [
            self._context.get_expectation_suite(name)
            for name in self._context.list_expectation_suite_names()
        ]

    def _get_all_checkpoints(self) -> List[CheckpointConfig]:
        return [
            self._context.checkpoint_store.get_checkpoint(name=checkpoint_name, id=None)
            for checkpoint_name in self._context.list_checkpoints()
        ]

    def _get_all_profilers(self) -> List[RuleBasedProfilerConfig]:
        def round_trip_profiler_config(
            profiler_config: RuleBasedProfilerConfig,
        ) -> RuleBasedProfilerConfig:
            return ruleBasedProfilerConfigSchema.load(
                ruleBasedProfilerConfigSchema.dump(profiler_config)
            )

        return [
            round_trip_profiler_config(self._context.get_profiler(name).config)
            for name in self._context.list_profilers()
        ]

    def _get_all_validation_results(
        self,
    ) -> Dict[str, ExpectationSuiteValidationResult]:
        validation_results = {
            str(key): cast(
                ExpectationSuiteValidationResult,
                self._context.validations_store.get(key),
            )
            for key in self._context.validations_store.list_keys()
        }
        return validation_results


class ConfigurationBundleSchema(Schema):
    """Marshmallow Schema for the Configuration Bundle."""

    data_context_id = fields.String(allow_none=False, required=True)
    data_context_variables = fields.Nested(DataContextConfigSchema, allow_none=False)
    datasources = fields.List(
        fields.Nested(DatasourceConfigSchema, allow_none=True, required=True),
        required=True,
    )
    expectation_suites = fields.List(
        fields.Nested(ExpectationSuiteSchema, allow_none=True, required=True),
        required=True,
    )
    checkpoints = fields.List(
        fields.Nested(CheckpointConfigSchema, allow_none=True, required=True),
        required=True,
    )
    profilers = fields.List(
        fields.Nested(RuleBasedProfilerConfigSchema, allow_none=True, required=True),
        required=True,
    )
    validation_results = fields.Dict(
        keys=fields.String(
            required=True,
            allow_none=False,
        ),
        values=fields.Nested(
            ExpectationSuiteValidationResultSchema, allow_none=True, required=True
        ),
        required=True,
    )

    @post_dump
    def clean_up(self, data, **kwargs) -> dict:
        data_context_variables = data.get("data_context_variables", {})
        data_context_variables.pop("anonymous_usage_statistics", None)
        return data


class ConfigurationBundleJsonSerializer:
    def __init__(self, schema: Schema) -> None:
        """

        Args:
            schema: Marshmallow schema defining raw serialized version of object.
        """
        self.schema = schema

    def serialize(self, obj: ConfigurationBundle) -> dict:
        """Serialize config to json dict.

        Args:
            obj: AbstractConfig object to serialize.

        Returns:
            Representation of object as a dict suitable for serializing to json.
        """
        config: dict = self.schema.dump(obj)

        json_serializable_dict: dict = convert_to_json_serializable(data=config)

        return json_serializable_dict



from __future__ import annotations

from typing import TYPE_CHECKING, Optional, Union

import great_expectations.exceptions as gx_exceptions
from great_expectations.core.id_dict import BatchSpec  # noqa: TCH001
from great_expectations.execution_engine.split_and_sample.data_sampler import (
    DataSampler,
)
from great_expectations.execution_engine.sqlalchemy_dialect import GXSqlDialect

try:
    import sqlalchemy as sa
except ImportError:
    sa = None

try:
    from sqlalchemy.engine import Dialect
    from sqlalchemy.sql import Selectable
    from sqlalchemy.sql.elements import BinaryExpression, BooleanClauseList
except ImportError:
    Selectable = None
    BinaryExpression = None
    BooleanClauseList = None
    Dialect = None


if TYPE_CHECKING:
    import sqlalchemy as sa  # noqa: TCH004

    from great_expectations.execution_engine import SqlAlchemyExecutionEngine


class SqlAlchemyDataSampler(DataSampler):
    """Sampling methods for data stores with SQL interfaces."""

    def sample_using_limit(
        self,
        execution_engine: SqlAlchemyExecutionEngine,
        batch_spec: BatchSpec,
        where_clause: Optional[Selectable] = None,
    ) -> Union[str, BinaryExpression, BooleanClauseList]:
        """Sample using a limit with configuration provided via the batch_spec.

        Note: where_clause needs to be included at this stage since SqlAlchemy's semantics
        for LIMIT are different than normal WHERE clauses.

        Also this requires an engine to find the dialect since certain databases require
        different handling.

        Args:
            execution_engine: Engine used to connect to the database.
            batch_spec: Batch specification describing the batch of interest.
            where_clause: Optional clause used in WHERE clause. Typically generated by a splitter.

        Returns:
            A query as a string or sqlalchemy object.
        """

        # Split clause should be permissive of all values if not supplied.
        if where_clause is None:
            if execution_engine.dialect_name == GXSqlDialect.SQLITE:
                where_clause = sa.text("1 = 1")
            else:
                where_clause = sa.true()

        table_name: str = batch_spec["table_name"]

        # SQLalchemy's semantics for LIMIT are different than normal WHERE clauses,
        # so the business logic for building the query needs to be different.
        dialect_name: str = execution_engine.dialect_name
        if dialect_name == GXSqlDialect.ORACLE:
            # TODO: AJB 20220429 WARNING THIS oracle dialect METHOD IS NOT COVERED BY TESTS
            # limit doesn't compile properly for oracle so we will append rownum to query string later
            raw_query: Selectable = (
                sa.select("*")
                .select_from(
                    sa.table(table_name, schema=batch_spec.get("schema_name", None))
                )
                .where(where_clause)
            )
            query: str = str(
                raw_query.compile(
                    dialect=execution_engine.dialect,
                    compile_kwargs={"literal_binds": True},
                )
            )
            query += "\nAND ROWNUM <= %d" % batch_spec["sampling_kwargs"]["n"]
            return query
        elif dialect_name == GXSqlDialect.MSSQL:
            # Note that this code path exists because the limit parameter is not getting rendered
            # successfully in the resulting mssql query.
            selectable_query: Selectable = (
                sa.select("*")
                .select_from(
                    sa.table(table_name, schema=batch_spec.get("schema_name", None))
                )
                .where(where_clause)
                .limit(batch_spec["sampling_kwargs"]["n"])
            )
            string_of_query: str = str(
                selectable_query.compile(
                    dialect=execution_engine.dialect,
                    compile_kwargs={"literal_binds": True},
                )
            )
            n: Union[str, int] = batch_spec["sampling_kwargs"]["n"]
            self._validate_mssql_limit_param(n)
            # This string replacement is here because the limit parameter is not substituted during query.compile()
            string_of_query = string_of_query.replace("?", str(n))
            return string_of_query
        else:
            return (
                sa.select("*")
                .select_from(
                    sa.table(table_name, schema=batch_spec.get("schema_name", None))
                )
                .where(where_clause)
                .limit(batch_spec["sampling_kwargs"]["n"])
            )

    @staticmethod
    def _validate_mssql_limit_param(n: Union[str, int]) -> None:
        """Validate that the mssql limit param is passed as an int or a string representation of an int.

        Args:
            n: mssql limit parameter.

        Returns:
            None
        """
        if not isinstance(n, (str, int)):
            raise gx_exceptions.InvalidConfigError(
                "Please specify your sampling kwargs 'n' parameter as a string or int."
            )
        if isinstance(n, str) and not n.isdigit():
            raise gx_exceptions.InvalidConfigError(
                "If specifying your sampling kwargs 'n' parameter as a string please ensure it is "
                "parseable as an integer."
            )

    @staticmethod
    def sample_using_random(
        execution_engine: SqlAlchemyExecutionEngine,
        batch_spec: BatchSpec,
        where_clause: Optional[Selectable] = None,
    ) -> Selectable:
        """Sample using random data with configuration provided via the batch_spec.

        Note: where_clause needs to be included at this stage since we use the where clause
        to determine the total number of rows to use in determining the rows returned in the
        sample fraction.

        Args:
            execution_engine: Engine used to connect to the database.
            batch_spec: Batch specification describing the batch of interest.
            where_clause: Optional clause used in WHERE clause. Typically generated by a splitter.

        Returns:
            Sqlalchemy selectable.
        """

        # TODO: AJB 20220429 WARNING THIS METHOD IS NOT COVERED BY TESTS

        table_name: str = batch_spec["table_name"]

        num_rows: int = execution_engine.engine.execute(
            sa.select([sa.func.count()])
            .select_from(
                sa.table(table_name, schema=batch_spec.get("schema_name", None))
            )
            .where(where_clause)
        ).scalar()
        p: float = batch_spec["sampling_kwargs"]["p"] or 1.0
        sample_size: int = round(p * num_rows)
        return (
            sa.select("*")
            .select_from(
                sa.table(table_name, schema=batch_spec.get("schema_name", None))
            )
            .where(where_clause)
            .order_by(sa.func.random())
            .limit(sample_size)
        )

    def sample_using_mod(
        self,
        batch_spec: BatchSpec,
    ) -> Selectable:
        """Take the mod of named column, and only keep rows that match the given value.

        Args:
            batch_spec: should contain keys `column_name`, `mod` and `value`

        Returns:
            Sampled selectable

        Raises:
            SamplerError
        """
        self.verify_batch_spec_sampling_kwargs_exists(batch_spec)
        self.verify_batch_spec_sampling_kwargs_key_exists("column_name", batch_spec)
        self.verify_batch_spec_sampling_kwargs_key_exists("mod", batch_spec)
        self.verify_batch_spec_sampling_kwargs_key_exists("value", batch_spec)
        column_name: str = self.get_sampling_kwargs_value_or_default(
            batch_spec, "column_name"
        )
        mod: int = self.get_sampling_kwargs_value_or_default(batch_spec, "mod")
        value: int = self.get_sampling_kwargs_value_or_default(batch_spec, "value")

        return sa.column(column_name) % mod == value

    def sample_using_a_list(
        self,
        batch_spec: BatchSpec,
    ) -> Selectable:
        """Match the values in the named column against value_list, and only keep the matches.

        Args:
            batch_spec: should contain keys `column_name` and `value_list`

        Returns:
            Sampled selectable

        Raises:
            SamplerError
        """
        self.verify_batch_spec_sampling_kwargs_exists(batch_spec)
        self.verify_batch_spec_sampling_kwargs_key_exists("column_name", batch_spec)
        self.verify_batch_spec_sampling_kwargs_key_exists("value_list", batch_spec)
        column_name: str = self.get_sampling_kwargs_value_or_default(
            batch_spec, "column_name"
        )
        value_list: list = self.get_sampling_kwargs_value_or_default(
            batch_spec, "value_list"
        )
        return sa.column(column_name).in_(value_list)

    def sample_using_md5(
        self,
        batch_spec: BatchSpec,
    ) -> Selectable:
        """Hash the values in the named column using md5, and only keep rows that match the given hash_value.

        Args:
            batch_spec: should contain keys `column_name` and optionally `hash_digits`
                (default is 1 if not provided), `hash_value` (default is "f" if not provided)

        Returns:
            Sampled selectable

        Raises:
            SamplerError
        """
        self.verify_batch_spec_sampling_kwargs_exists(batch_spec)
        self.verify_batch_spec_sampling_kwargs_key_exists("column_name", batch_spec)
        column_name: str = self.get_sampling_kwargs_value_or_default(
            batch_spec, "column_name"
        )
        hash_digits: int = self.get_sampling_kwargs_value_or_default(
            batch_spec=batch_spec, sampling_kwargs_key="hash_digits", default_value=1
        )
        hash_value: str = self.get_sampling_kwargs_value_or_default(
            batch_spec=batch_spec, sampling_kwargs_key="hash_value", default_value="f"
        )

        return (
            sa.func.right(
                sa.func.md5(sa.cast(sa.column(column_name), sa.Text)), hash_digits
            )
            == hash_value
        )
